package com.hand.hls.dp.controllers;

import com.hand.hap.core.IRequest;
import com.hand.hap.core.impl.RequestHelper;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import com.hand.hls.dp.service.SysDpDownloadLogService;
import com.hand.hls.dp.service.SysDpExecuteHistoryService;
import com.hand.hls.dp.util.IPUtils;

import leaf.bean.LeafRequestData;
import org.apache.commons.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;

/**
 * @author <a href="mailto:zhangnet14@gmail.com">Spencer Chang</a>
 * @date 2020/4/18 15:31
 * @since
 */

@RestController
public class SysDpExecuteHistoryController extends BaseController {

    @Autowired
    private SysDpExecuteHistoryService service;
    @Autowired
    private SysDpDownloadLogService downloadLogService;

    @PostMapping(value = "/sys/dp/query")
    public ResponseData query(@ModelAttribute(LEAF_PARAM_NAME) LeafRequestData requestData, @RequestParam("ck") int ck,
                              HttpServletRequest request, HttpServletResponse response) throws Exception{
        IRequest requestContext = createRequestContext(request);
        RequestHelper.setCurrentRequest(requestContext);
        final String ip = IPUtils.getIpAddress(request);
        final String param = (String) requestData.get("parameter");
        final String dpContext = URLDecoder.decode(new String(Base64.decodeBase64(param), StandardCharsets.UTF_8),"UTF-8");
        if (ck == 0){
            service.query4Verify(requestContext, ip, dpContext);
        }else if(ck == 1){
            request.setAttribute("_NO_PAGE_","Y");
            final String context = new String(dpContext.getBytes(StandardCharsets.ISO_8859_1), StandardCharsets.UTF_8);
            service.query(requestContext, response, ip, context);
        }
        return new ResponseData();
    }
    @PostMapping(value = "/sys/dp/insert")
    public ResponseData insert(@ModelAttribute(LEAF_PARAM_NAME) LeafRequestData requestData, HttpServletRequest request) throws Exception{
        IRequest requestContext = createRequestContext(request);
        RequestHelper.setCurrentRequest(requestContext);
        final String ip = IPUtils.getIpAddress(request);
        final String dpContext = (String) requestData.get("parameter");
        return service.insert(requestContext, ip, dpContext);
    }
    @PostMapping(value = "/sys/dp/update")
    public ResponseData update(@ModelAttribute(LEAF_PARAM_NAME) LeafRequestData requestData, HttpServletRequest request) throws Exception{
        IRequest requestContext = createRequestContext(request);
        RequestHelper.setCurrentRequest(requestContext);
        final String ip = IPUtils.getIpAddress(request);
        final String dpContext = (String) requestData.get("parameter");
        return service.update(requestContext, ip, dpContext);
    }
    @PostMapping(value = "/sys/dp/delete")
    public ResponseData delete(@ModelAttribute(LEAF_PARAM_NAME) LeafRequestData requestData, HttpServletRequest request) throws Exception{
        IRequest requestContext = createRequestContext(request);
        RequestHelper.setCurrentRequest(requestContext);
        final String ip = IPUtils.getIpAddress(request);
        final String dpContext = (String) requestData.get("parameter");
        return service.delete(requestContext, ip, dpContext);
    }
    @PostMapping(value = "/sys/dp/download/log")
    public void downloadLog(HttpServletRequest request, HttpServletResponse response) throws Exception{
        IRequest requestContext = createRequestContext(request);
        downloadLogService.download(requestContext, request, response);
    }

    @PostMapping(value = "/sys/dp/ddl")
    public void ddl(@ModelAttribute(LEAF_PARAM_NAME) LeafRequestData requestData, HttpServletRequest request) throws Exception{
        IRequest requestContext = createRequestContext(request);
        RequestHelper.setCurrentRequest(requestContext);
        final String ip = IPUtils.getIpAddress(request);
        final String dpContext = (String) requestData.get("parameter");
        service.ddl(requestContext, ip, dpContext);
    }
}
