package com.hand.hls.dp.util;

import org.apache.commons.lang3.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.math.BigInteger;
import java.net.InetAddress;
import java.nio.Buffer;
import java.util.Arrays;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author <a href="mailto:zhangnet14@gmail.com">Spencer Chang</a>
 * @date 2020/4/18 15:41
 * @since
 */
public class IPUtils {
    private static final String IPV4_LOCAL_HOST = "127.0.0.1";
    private static final String IPV6_LOCAL_HOST = "0:0:0:0:0:0:0:1";

    public static String getIpAddress(HttpServletRequest request) {
        if (Objects.isNull(request)) {
            return "";
        }
        // X-Forwarded-For：Squid 服务代理
        String ip = request.getHeader("x-forwarded-for");
        if (Objects.isNull(ip) || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            // Proxy-Client-IP：apache 服务代理
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (Objects.isNull(ip) || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            // WL-Proxy-Client-IP：weblogic 服务代理
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (Objects.isNull(ip) || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_CLIENT_IP");
        }
        if (Objects.isNull(ip) || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            // X-Real-IP：nginx服务代理
            ip = request.getHeader("X-Real-IP");
        }
        if (Objects.isNull(ip) || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_X_FORWARDED_FOR");
        }
        if (Objects.isNull(ip) || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
            // 本地局域网放行
            if (Objects.equals(IPV4_LOCAL_HOST, ip) || Objects.equals(IPV6_LOCAL_HOST, ip)) {
                // 根据网卡获取本机配置的ip
                InetAddress inetAddress = null;
                try {
                    inetAddress = InetAddress.getLocalHost();
                    ip = inetAddress.getHostAddress();
                } catch (Exception e) {
                    // 默认
                    ip = IPV4_LOCAL_HOST;
                }
            }
        }
        // 对于通过多个代理的情况，第一个IP为客户端真实IP,多个IP按照','分割
        // "***.***.***.***".length() = 15
        if (Objects.nonNull(ip) && ip.length() > 15) {
            if (ip.indexOf(",") > 0) {
                ip = ip.substring(0, ip.indexOf(","));
            }
        }
        return extractString(ip);
    }
    // 获取本机ip
    public static String getIpAddressLocal() {
        String ip = IPV4_LOCAL_HOST;
        // 根据网卡获取本机配置的ip
        InetAddress inetAddress = null;
        try {
            inetAddress = InetAddress.getLocalHost();
            ip = inetAddress.getHostAddress();
        } catch (Exception e) {
            // ignore
        }
        return ip;
    }

    /**
     * 获取访问主机的网段（4位）
     */
    private static String extractString(String s) {
        if (StringUtils.isEmpty(s)) {
            return "";
        }
        StringBuilder retVal = new StringBuilder();
        String[] split = s.split("\\.");
        for (int i = 0; i < 4; i++) {
            retVal.append(split[i]);
            if (i < 3) {
                retVal.append(".");
            }
        }
        return retVal.toString();
    }
}
